package com.jpsoft.smart.modules.base.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONObject;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.modules.base.dao.PersonDeviceFilterLogDAO;
import com.jpsoft.smart.modules.base.dao.PersonDeviceLogDAO;
import com.jpsoft.smart.modules.base.entity.PersonDeviceFilterLog;
import com.jpsoft.smart.modules.base.service.PersonDeviceFilterLogService;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.LApiUtil;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import sun.misc.BASE64Decoder;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2020-3-20 9:14
 */
@Slf4j
@Transactional
@Component(value="personDeviceFilterLogService")
public class PersonDeviceFilterLogServiceImpl implements PersonDeviceFilterLogService {

    @Autowired
    private OSSConfig ossConfig;

    @Resource(name="personDeviceFilterLogDAO")
    private PersonDeviceFilterLogDAO personDeviceFilterLogDAO;

    @Override
    public PersonDeviceFilterLog get(Long id) {
        // TODO Auto-generated method stub
        return personDeviceFilterLogDAO.get(id);
    }

    @Override
    public void deviceInsertFilterLog(String deviceNo, BigDecimal temperature, JSONObject faceImageJson, JSONObject libMatInfoListJson, JSONObject matchPersonInfo, Date date) {

        try{


        String retFileUrl = "";
        try{

            BASE64Decoder decoder = new BASE64Decoder();

            byte[] imgData = decoder.decodeBuffer(faceImageJson.getStr("Data"));

            for (int i = 0; i < imgData.length; ++i) {
                if (imgData[i] < 0) {// 调整异常数据
                    imgData[i] += 256;
                }
            }

            ByteArrayInputStream inputStream = new ByteArrayInputStream(imgData);



            retFileUrl = OSSUtil.upload(ossConfig,"/devicePersonLog",faceImageJson.getStr("Name"), inputStream);
        }catch (Exception e){
            e.printStackTrace();
        }

        //记录的时间段
        String timeZones = DateUtil.format(date, "yyyy-MM-dd-HH");
        Integer personId = LApiUtil.getPersonCode(libMatInfoListJson,matchPersonInfo);
        if (personId !=0 && temperature.compareTo(BigDecimal.ZERO)==1){
            PersonDeviceFilterLog personDeviceFilterLog = personDeviceFilterLogDAO.findByPersonIdAndDeviceNoAndTimeZones(personId,deviceNo,timeZones);

            if (personDeviceFilterLog != null){
                BigDecimal oldTemper = personDeviceFilterLog.getTemperature();
                personDeviceFilterLog.setUpdateTime(new Date());
                personDeviceFilterLog.setTemperature(temperature);
                personDeviceFilterLog.setRecordTime(date);
                personDeviceFilterLog.setFaceImage(retFileUrl);
                personDeviceFilterLog.setCreateTime(new Date());

                if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchStatus"))){
                    personDeviceFilterLog.setMatchStatus(libMatInfoListJson.getInt("MatchStatus"));
                    personDeviceFilterLog.setMatchMsg(LApiUtil.getMatchMsg(libMatInfoListJson.getInt("MatchStatus")));
                    personDeviceFilterLog.setMatchFaceId(libMatInfoListJson.getInt("MatchFaceID"));
                }else {
                    personDeviceFilterLog.setMatchStatus(0);
                    personDeviceFilterLog.setMatchFaceId(0);
                }

                if (!LApiUtil.compareToInterval(oldTemper,36,37)){
                    if (LApiUtil.compareToInterval(temperature,36,37)){

                        personDeviceFilterLogDAO.update(personDeviceFilterLog);
                    }else {
                        if (temperature.compareTo(oldTemper)==-1){
                            personDeviceFilterLogDAO.update(personDeviceFilterLog);
                        }
                    }
                }
            }else {
                PersonDeviceFilterLog personDeviceFilterLog1 = new PersonDeviceFilterLog();
                personDeviceFilterLog1.setDeviceNo(deviceNo);
                personDeviceFilterLog1.setPersonId(personId);
                personDeviceFilterLog1.setTemperature(temperature);
                if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchStatus"))){
                    personDeviceFilterLog1.setMatchStatus(libMatInfoListJson.getInt("MatchStatus"));
                    personDeviceFilterLog1.setMatchMsg(LApiUtil.getMatchMsg(libMatInfoListJson.getInt("MatchStatus")));
                    personDeviceFilterLog1.setMatchFaceId(libMatInfoListJson.getInt("MatchFaceID"));
                }else {
                    personDeviceFilterLog1.setMatchStatus(0);
                    personDeviceFilterLog1.setMatchFaceId(0);
                }
                personDeviceFilterLog1.setFaceImage(retFileUrl);
                personDeviceFilterLog1.setRecordTime(date);
                personDeviceFilterLog1.setTimeZones(timeZones);
                personDeviceFilterLog1.setDelFlag(false);
                personDeviceFilterLog1.setCreateTime(new Date());

                personDeviceFilterLogDAO.insert(personDeviceFilterLog1);
            }
        }

        }catch (Exception e){

            log.error(e.getMessage());
        }



    }


    @Override
    public  Page<PersonDeviceFilterLog> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, boolean count, List<Sort> sortList){
        Page<PersonDeviceFilterLog> page = PageHelper.startPage(pageNumber,pageSize,count).doSelectPage(()->{
            personDeviceFilterLogDAO.search(searchParams,sortList);
        });

        return page;
    }

    @Override
    public List<PersonDeviceFilterLog> findByPersonAndDate(Long personId,String startDate,String endDate){
        return personDeviceFilterLogDAO.findByPersonAndDate(personId,startDate,endDate);
    }


    @Override
    public List<PersonDeviceFilterLog> temperatureNormalStatistics(BigDecimal temperature,String startDate,String endDate){
        return personDeviceFilterLogDAO.temperatureNormalStatistics(temperature,startDate,endDate);
    }

    @Override
    public List<PersonDeviceFilterLog> temperatureUnusualStatistics(BigDecimal temperature,String startDate,String endDate){
        return personDeviceFilterLogDAO.temperatureUnusualStatistics(temperature,startDate,endDate);
    }


    @Override
    public List<PersonDeviceFilterLog> list(){
        return personDeviceFilterLogDAO.list();
    }

    @Override
    public List<Map<String, Object>> unusualStatistics(BigDecimal temperature,String startDate,String endDate){
        return personDeviceFilterLogDAO.unusualStatistics(temperature,startDate,endDate);
    }
}
