package com.jpsoft.smart.lapi.handler;

import cn.hutool.json.JSONObject;
import com.jpsoft.smart.lapi.dto.PersonVerification;
import com.jpsoft.smart.modules.base.service.PersonDeviceFilterLogService;
import com.jpsoft.smart.modules.base.service.PersonDeviceLogService;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

@Slf4j
@Component
@ChannelHandler.Sharable
public class PersonVerificationHandler extends SimpleChannelInboundHandler<PersonVerification> {

    @Autowired
    private PersonDeviceLogService personDeviceLogService;

    @Autowired
    private PersonDeviceFilterLogService personDeviceFilterLogService;

    @Override
    protected void channelRead0(ChannelHandlerContext channelHandlerContext, PersonVerification personVerification) throws Exception {



        log.warn("人员记录");
        JSONObject jsonObject = personVerification.getJsonObject();




        JSONObject faceJson = new JSONObject(jsonObject.getStr("FaceInfoList").substring(1,jsonObject.getStr("FaceInfoList").length()-1));


        Long timestamp = faceJson.getLong("Timestamp");
        if (timestamp.toString().length()<13){
             double timestampDouble = timestamp *  Math.pow(10, 13 - timestamp.toString().length());
             timestamp =new Double(timestampDouble).longValue() ;
        }

        log.warn(String.valueOf(timestamp));
        Date date = new Date(timestamp);






        //设备序列号
        String deviceNo = jsonObject.getStr("DeviceCode");

        //人员温度信息
        BigDecimal temperature = faceJson.getBigDecimal("Temperature");

        //人员抓拍信息
        JSONObject faceImageJson = faceJson.getJSONObject("FaceImage");

        //匹配状态信息
        JSONObject libMatInfoListJson = new JSONObject();
        if (StringUtils.isNotBlank(jsonObject.getStr("LibMatInfoList").substring(1,jsonObject.getStr("LibMatInfoList").length()-1))){
             libMatInfoListJson = new JSONObject(jsonObject.getStr("LibMatInfoList").substring(1,jsonObject.getStr("LibMatInfoList").length()-1));
        }


        //匹配人员信息
        JSONObject matchPersonInfo = new JSONObject();
        if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchPersonInfo"))){
            matchPersonInfo = libMatInfoListJson.getJSONObject("MatchPersonInfo");
        }



        log.warn("人员记录写入");
        HashMap map = new HashMap<>();
        map.put("deviceNo",deviceNo);
        map.put("temperature",temperature);
        map.put("libMatInfoListJson",libMatInfoListJson);
        map.put("matchPersonInfo",matchPersonInfo);
        log.warn(map.toString());


        personDeviceLogService.deviceInsertLog(deviceNo,temperature,faceImageJson,libMatInfoListJson,matchPersonInfo,date);

        personDeviceFilterLogService.deviceInsertFilterLog(deviceNo,temperature,faceImageJson,libMatInfoListJson,matchPersonInfo,date);






    }
}
