package com.jpsoft.smart.modules.mobile.controller;

import com.github.pagehelper.Page;
import com.github.pagehelper.util.StringUtil;
import com.google.common.collect.Lists;
import com.jpsoft.smart.modules.base.entity.*;
import com.jpsoft.smart.modules.base.service.*;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.utils.StringUtils;
import com.jpsoft.smart.modules.sys.entity.User;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/mobile/personDeviceLogApi")
public class PersonDeviceLogApiController {
    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private PersonDeviceLogService personDeviceLogService;

    @Autowired
    private PersonDeviceFilterLogService personDeviceFilterLogService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private DeviceInfoService deviceInfoService;

    @PostMapping("queryTemperatureRawRecordList")
    @ApiOperation(value="根据实时数据查看企业人员温度数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name="queryDate",value = "查询日期",required = true,paramType = "form"),
            @ApiImplicitParam(name="companyId",value = "单位编号",paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map> queryTemperatureRawRecordList(
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date queryDate,
            String companyId,
            String token,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        Page<Map> recordPageList = new Page<>();

        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo admin = personInfoService.get(Long.valueOf(subject));

            Map<String, Object> personSearchParam = new HashMap<>();

            if (StringUtils.isNotEmpty(companyId) && companyInfoService.exist(companyId)) {
                personSearchParam.put("companyId", companyId);
            }
            else{
                personSearchParam.put("companyId", admin.getCompanyId());
            }

            List<Sort> sortList1 = new ArrayList<>();
            sortList1.add(new Sort("id_","asc"));

            Page<PersonInfo> personPageList = personInfoService.pageSearch(personSearchParam,pageIndex,pageSize,true,sortList1);

            DateTime startTime = new DateTime(queryDate);

            for (PersonInfo person : personPageList) {
                Map<String, Object> recordMap = new HashMap<>();

                recordMap.put("name", person.getName());
                recordMap.put("faceImageUrl", person.getFaceImageUrl());

                Map<String, Object> searchParams = new HashMap<>();

                searchParams.put("personId", person.getId());
                searchParams.put("beginTime", startTime.toDate());
                searchParams.put("endTime", startTime.plusDays(1).toDate());

                List<Sort> sortList = new ArrayList<>();
                sortList.add(new Sort("record_time", "asc"));

                Page<PersonDeviceLog> page = personDeviceLogService.simplePageSearch(searchParams, 1, 100, false, sortList);

                Map<Integer, List> hourMap = new TreeMap<>();

                //收集每个人每小时的测量温度
                for (PersonDeviceLog personDeviceLog : page) {
                    DateTime recordTime = new DateTime(personDeviceLog.getRecordTime());

                    int hour = recordTime.getHourOfDay();

                    if (hourMap.containsKey(hour)) {
                        hourMap.get(hour).add(personDeviceLog);
                    } else {
                        List<PersonDeviceLog> logList = new ArrayList<>();
                        logList.add(personDeviceLog);
                        hourMap.put(hour, logList);
                    }
                }

                //每个小时取一条温度数据小于37的即可，如果没有则取最低的
                BigDecimal stdLower = new BigDecimal(36);
                BigDecimal stdUpper = new BigDecimal(37);

                Map<Integer, PersonDeviceLog> hourFilterMap = new TreeMap<>();

                for (Integer hour : hourMap.keySet()) {
                    List<PersonDeviceLog> logList = hourMap.get(hour);

                    List<PersonDeviceLog> filterList = logList.stream()
                            //查询是否有36度至37度
                            .filter((log) -> {
                                if (log.getTemperature().compareTo(stdLower) >= 0
                                        && log.getTemperature().compareTo(stdUpper) <= 0) {
                                    return true;
                                } else {
                                    return false;
                                }
                            })
                            //体温由低到高排列
                            .sorted(Comparator.comparing(PersonDeviceLog::getTemperature)).collect(Collectors.toList());

                    if (filterList.size() > 0) {
                        hourFilterMap.put(hour, filterList.get(0));
                    } else {
                        //如果不存在，则取最低的一条
                        Optional<PersonDeviceLog> optional = logList.stream()
                                .sorted(Comparator.comparing(PersonDeviceLog::getTemperature)).findFirst();

                        if (optional.isPresent()) {
                            hourFilterMap.put(hour, optional.get());
                        }
                    }
                }

                SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");

                List<Map> personDeviceLogs = hourFilterMap.keySet().stream()
                        .map((key) -> hourFilterMap.get(key))
                        .filter((log)->log.getTemperature().compareTo(BigDecimal.ZERO)>0)
                        .sorted(Comparator.comparing(PersonDeviceLog::getRecordTime))
                        .map((log)->{
                            Map<String,Object> map = new HashMap<>();
                            map.put("temperature",log.getTemperature());
                            map.put("recordTime",sdf.format(log.getRecordTime()));

                            return map;
                        })
                        .collect(Collectors.toList());

                recordMap.put("list", personDeviceLogs);

                recordPageList.add(recordMap);
            }

            recordPageList.setTotal(personPageList.getTotal());
            recordPageList.setPages(personPageList.getPages());
            recordPageList.setPageNum(personPageList.getPageNum());
            recordPageList.setPageSize(personPageList.getPageSize());

            messageResult.setResult(true);
            messageResult.setData(PojoUtils.pageWrapper(recordPageList));
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("queryTemperatureRecordListByEnt")
    @ApiOperation(value="根据过滤后考勤记录统计单位员工温度数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name="queryDate",value = "查询日期",required = true,paramType = "form"),
            @ApiImplicitParam(name="companyId",value = "单位编号",paramType = "form"),
            @ApiImplicitParam(name="filter",value = "是否过滤（1-显示异常数据,0-显示全部)",paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map> queryTemperatureRecordListByEnt(
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date queryDate,
            String companyId,
            String token,
            String filter,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        Page<Map> recordPageList = new Page<>();

        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo admin = personInfoService.get(Long.valueOf(subject));

            Map<String, Object> personSearchParam = new HashMap<>();

            if (StringUtils.isNotEmpty(companyId) && companyInfoService.exist(companyId)) {
                personSearchParam.put("companyId", companyId);
            }
            else{
                personSearchParam.put("companyId", admin.getCompanyId());
            }

            if (StringUtils.isNotEmpty(filter) && "1".equals(filter)){
                personSearchParam.put("abnormalDate",queryDate);
            }

            List<Sort> sortList1 = new ArrayList<>();
            sortList1.add(new Sort("id_","asc"));

            Page<PersonInfo> personPageList = personInfoService.pageSearch(personSearchParam,pageIndex,pageSize,true,sortList1);

            DateTime startTime = new DateTime(queryDate);

            for (PersonInfo person : personPageList) {
                Map<String, Object> recordMap = new HashMap<>();

                recordMap.put("name", person.getName());
                recordMap.put("faceImageUrl", person.getFaceImageUrl());

                Map<String, Object> searchParams = new HashMap<>();

                searchParams.put("personId", person.getId());
                searchParams.put("beginTime", startTime.toDate());
                searchParams.put("endTime", startTime.plusDays(1).toDate());

                if (StringUtils.isNotEmpty(filter) && "1".equals(filter)){
                    searchParams.put("abnormalFilter","1");
                }

                List<Sort> sortList = new ArrayList<>();
                sortList.add(new Sort("record_time", "asc"));

                Page<PersonDeviceFilterLog> page = personDeviceFilterLogService.pageSearch(searchParams,1,24,false,sortList);

                SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");

                List<Map> personDeviceLogs = page.stream()
                        .filter((log)->log.getTemperature().compareTo(BigDecimal.ZERO)>0)
                        .map((log)->{
                            Map<String,Object> map = new HashMap<>();
                            map.put("temperature",log.getTemperature());
                            map.put("recordTime",sdf.format(log.getRecordTime()));

                            return map;
                        })
                        .collect(Collectors.toList());

                recordMap.put("list", personDeviceLogs);

                recordPageList.add(recordMap);
            }

            recordPageList.setTotal(personPageList.getTotal());
            recordPageList.setPages(personPageList.getPages());
            recordPageList.setPageNum(personPageList.getPageNum());
            recordPageList.setPageSize(personPageList.getPageSize());

            messageResult.setResult(true);
            messageResult.setData(PojoUtils.pageWrapper(recordPageList));
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("queryCompanyList")
    @ApiOperation(value="查询企业列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<List> queryCompanyList(String token,String subject){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(Long.valueOf(subject));

            CompanyInfo companyInfo = companyInfoService.get(personInfo.getCompanyId());

            List<CompanyInfo> companyList = companyInfoService.findByCompanyCode(companyInfo.getCode() + "%");

            messageResult.setData(companyList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setMessage(ex.getMessage());
            messageResult.setResult(false);
        }

        return messageResult;
    }


    @PostMapping("listByPersonIdFourteenDays")
    @ApiOperation(value="查询个人用户14天内的日志台账")
    @ApiImplicitParams({
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<List> listByPersonIdFourteenDays(String token,@RequestAttribute String subject){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            Date now = new Date();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String endDate = sdf.format(now);
            Calendar rightNow = Calendar.getInstance();
            rightNow.setTime(now);
            rightNow.add(Calendar.DATE, -14);
            Date dt1 = rightNow.getTime();
            String startDate = sdf.format(dt1);

            PersonInfo person = personInfoService.get(Long.valueOf(subject));

            List<PersonDeviceFilterLog> personDeviceLogList = personDeviceFilterLogService.findByPersonAndDate(person.getId(),startDate,endDate);

            messageResult.setData(personDeviceLogList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setMessage(ex.getMessage());
            messageResult.setResult(false);
        }

        return messageResult;
    }


    @ApiOperation(value="移动端台账日志列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map> pageList(
            String token,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.record_time","desc"));

        PersonInfo person = personInfoService.get(Long.valueOf(subject));

        searchParams.put("personId",person.getId());

        Page<PersonDeviceFilterLog> page = personDeviceFilterLogService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="体温台账饼图统计")
    @RequestMapping(value = "temperaturePieChart",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="startTime",value = "开始时间(格式：yyyy-MM-dd)",paramType = "form"),
            @ApiImplicitParam(name="endTime",value = "截止时间(格式：yyyy-MM-dd)",paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<List<Map>> temperaturePieChart(
            String startTime,String endTime,
            String token, @RequestAttribute String subject){
        MessageResult<List<Map>> msgResult = new MessageResult<>();
        List<Map> listMap = new ArrayList<>();
        try{
            String startDate = "";
            String endDate = "";
            //37度以上为异常体温
            BigDecimal bd = new BigDecimal(37);

            if(StringUtil.isNotEmpty(startTime)&&StringUtil.isNotEmpty(endTime)) {
                startDate = startTime;
                endDate = endTime;
            }
            else{
                Date now = new Date();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

                Calendar rightNow = Calendar.getInstance();
                rightNow.setTime(now);
                rightNow.add(Calendar.DATE, -14);

                Calendar now1 = Calendar.getInstance();
                now1.setTime(now);
                now1.add(Calendar.DATE, 1);

                Date dt1 = rightNow.getTime();
                Date dt2 = now1.getTime();
                startDate = sdf.format(dt1);
                endDate = sdf.format(dt2);
            }

            List<PersonDeviceFilterLog> normalPersonDeviceFilterLogList = personDeviceFilterLogService.temperatureNormalStatistics(bd,startDate,endDate);
            List<PersonDeviceFilterLog> unusualPersonDeviceFilterLogList = personDeviceFilterLogService.temperatureUnusualStatistics(bd,startDate,endDate);

            Map<String,Object> map1 = new HashMap();

            map1.put("name","体温正常");
            map1.put("value",normalPersonDeviceFilterLogList.size());

            listMap.add(map1);

            Map<String,Object> map2 = new HashMap();

            map2.put("name","体温异常");
            map2.put("value",unusualPersonDeviceFilterLogList.size());

            listMap.add(map2);

            msgResult.setResult(true);
            msgResult.setData(listMap);
        }
        catch (Exception ex){
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }


    @ApiOperation(value="体温台账统计")
    @RequestMapping(value = "temperatureStatistics",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<List<Map>> temperatureStatistics(
            String token, @RequestAttribute String subject){
        MessageResult<List<Map>> msgResult = new MessageResult<>();
        List<Map> listMap = new ArrayList<>();

        try{
            List<PersonDeviceFilterLog> personDeviceFilterLogList = personDeviceFilterLogService.list();
            Map map = new HashMap();
            map.put("detected",personDeviceFilterLogList.size());
            List<CompanyInfo> companyList = companyInfoService.list();

            map.put("companyCount",companyList.size());

            List<PersonInfo> personInfo = personInfoService.list();

            map.put("personCount",personInfo.size());
            listMap.add(map);

            msgResult.setResult(true);
            msgResult.setData(listMap);

        }
        catch (Exception ex){
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }


    @ApiOperation(value="异常统计")
    @RequestMapping(value = "unusualStatistics",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="startTime",value = "开始时间(格式：yyyy-MM-dd)",paramType = "form"),
            @ApiImplicitParam(name="endTime",value = "截止时间(格式：yyyy-MM-dd)",paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map<String, Object>> unusualStatistics(
            String startTime,String endTime,
            String token, @RequestAttribute String subject) {
        MessageResult<Map<String, Object>> msgResult = new MessageResult<>();

        Map<String, Object> map = new HashMap<>();

        try{
            String startDate = "";
            String endDate = "";
            //37度以上为异常体温
            BigDecimal bd = new BigDecimal(37);

            if(StringUtil.isNotEmpty(startTime)&&StringUtil.isNotEmpty(endTime)) {
                startDate = startTime;
                endDate = endTime;
            }
            else{
                Date now = new Date();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

                Calendar rightNow = Calendar.getInstance();
                rightNow.setTime(now);
                rightNow.add(Calendar.DATE, -14);

                Calendar now1 = Calendar.getInstance();
                now1.setTime(now);
                now1.add(Calendar.DATE, 1);

                Date dt1 = rightNow.getTime();
                Date dt2 = now1.getTime();
                startDate = sdf.format(dt1);
                endDate = sdf.format(dt2);
            }

            List<Map<String, Object>> mapList = personDeviceFilterLogService.unusualStatistics(bd,startDate,endDate);
            List<Map<String, Object>> list = new ArrayList<>();

            int companyTotalCount = 0;
            int personTotalCount = 0;

            for (Map<String, Object> curMap:mapList) {
                if(curMap.get("name")!=null){
                    list.add(curMap);
                    companyTotalCount++;
                    if(curMap.get("personCount")!=null){
                        String personCountStr = curMap.get("personCount").toString();
                        if(StringUtil.isNotEmpty(personCountStr)){
                            personTotalCount = personTotalCount + Integer.parseInt(personCountStr);
                        }
                    }

                }
            }

            map.put("companyTotalCount",companyTotalCount);
            map.put("personTotalCount",personTotalCount);
            map.put("list",list);

            msgResult.setResult(true);
            msgResult.setData(map);
        }
        catch (Exception ex){
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }


    @ApiOperation(value="获取设备列表")
    @RequestMapping(value = "getDeviceList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="token",value = "令牌",required = true,paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<List<DeviceInfo>> getDeviceList(String token, @RequestAttribute String subject) {
        MessageResult<List<DeviceInfo>> msgResult = new MessageResult<>();

        try{
            PersonInfo person = personInfoService.get(Long.valueOf(subject));

            String companyId = person.getCompanyId();

            List<DeviceInfo> DeviceInfoList = deviceInfoService.findByCompanyId(companyId);

            msgResult.setResult(true);
            msgResult.setData(DeviceInfoList);
        }
        catch (Exception ex){
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }
}
