package com.jpsoft.smart.modules.mobile.controller;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.domain.Person;
import com.github.pagehelper.Page;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.modules.base.entity.PersonDeviceRelation;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.service.PersonDeviceRelationService;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.utils.SMSUtil;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import com.jpsoft.smart.modules.lapi.vo.LapiMsgResult;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.sys.service.UserService;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.security.Key;
import java.util.*;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/mobile/personInfoApi")
public class PersonInfoApiController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private PersonDeviceRelationService personDeviceRelationService;

    @Autowired
    private ILapiService lapiService;

    @Autowired
    private ValueOperations<String, Object> valueOperations;

    @Autowired
    private UserService userService;

    @Autowired
    RabbitTemplate rabbitTemplate;

    @PostMapping("findByNameAndPhone")
    @ApiOperation(value="通过姓名和手机号查询人员(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="name",value = "姓名",required = true,paramType = "form"),
            @ApiImplicitParam(name = "phone",value = "电话号码", required = true,paramType="form")
    })
    public MessageResult<Long> findByNameAndPhone(String name,String phone){
        MessageResult<Long> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findByNameAndPhone(name, phone);

            if (personInfo==null){
                throw new Exception("您的信息暂未登记，请先联系管理人员!");
            }

            if (personInfo.getFaceEnabled()==null || !personInfo.getFaceEnabled()){
                throw new Exception("当前用户未被允许使用人脸识别!");
            }

            messageResult.setData(personInfo.getId());
            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("findByOpenId")
    @ApiOperation(value="通过openId查询人员(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="openId",value = "微信openId",required = true,paramType = "form")
    })
    public MessageResult<Map> findByOpenId(String openId){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findByOpenId(openId);

            if (personInfo==null){
                throw new Exception("当前用户不存在!");
            }

            Map<String,Object> dataMap = new HashMap<String, Object>();
            String token = createToken(personInfo.getId());

            dataMap.put("person",personInfo);
            dataMap.put("token", token);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("getVerifyCode")
    @ApiOperation(value="获取短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="personId",value = "人员编号",required = true,paramType = "form")
    })
    public MessageResult<String> getVerifyCode(Long personId){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
             PersonInfo personInfo = personInfoService.get(personId);

            if(personInfo==null){
                throw new Exception("人员编号不存在!");
            }

            String key = "SMS_" + personId;

            String  verifyCode =  (String)valueOperations.get(key);

            if(StringUtils.isEmpty(verifyCode)) {
                verifyCode = SMSUtil.generateNumberString(6);
                JSONObject verifyCodeJSON = new JSONObject();
                verifyCodeJSON.put("code", verifyCode);

                messageResult = SMSUtil.send(personInfo.getPhone(), "SMS_49390047", verifyCodeJSON.toString());

                //设置当前用户的验证码,1分钟内有效
                valueOperations.set(key, verifyCode, 1, TimeUnit.MINUTES);
            }

            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("validateCode")
    @ApiOperation(value="验证短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name="personId",value = "人员编号",required = true,paramType = "form"),
            @ApiImplicitParam(name="verifyCode",value = "验证码",required = true,paramType = "form"),
            @ApiImplicitParam(name="openId",value = "微信openId",paramType = "form")
    })
    public MessageResult<String> validateCode(Long personId,String verifyCode,String openId){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            String smsKey = "SMS_" + personId;

            String  beforeVerifyCode = (String)valueOperations.get(smsKey);

            if(StringUtils.isEmpty(beforeVerifyCode)) {
                throw new Exception("验证码已过期!");
            }

            if (!beforeVerifyCode.equals(verifyCode)){
                throw new Exception("验证码错误!");
            }

            if (StringUtils.isNotEmpty(openId)){
                PersonInfo personInfo = personInfoService.get(personId);
                personInfo.setOpenId(openId);
                personInfo.setUpdateTime(new Date());

                personInfoService.update(personInfo);
            }

            String token = createToken(personId);

            messageResult.setData(token);
            messageResult.setResult(true);
            messageResult.setCode(200);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    private String createToken(Long personId) {
        //token有效时间2小时
        byte[] privateKey = Base64.getDecoder().decode(jwtSecret);

        Date now = new Date();
        long expiration = now.getTime() + 3600 * 6000; //6个小时后，该客户端的token过期

        Key key = Keys.hmacShaKeyFor(privateKey);

        String token = Jwts.builder()
                .setSubject(personId + "")
                .signWith(key)
                .setExpiration(new Date(expiration))
                .compact();

        return "Bearer " + token;
    }

    @PostMapping("upload")
    @ApiOperation(value="人员照片上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name="photoName",value = "照片名称",required = true,paramType = "form"),
            @ApiImplicitParam(name = "photoFile",value = "员工照片", required = true,paramType="form", dataType = "__file"),
            @ApiImplicitParam(name="token",value = "令牌",required = false,paramType = "query"),
            @ApiImplicitParam(name="subject",value = "令牌",required = false,paramType = "query")
    })
    public MessageResult<String> upload(
            String photoName, MultipartFile photoFile,String token,@RequestAttribute  String subject){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(Long.valueOf(subject));

            if (personInfo==null){
                throw new Exception("人员信息不存在!");
            }

            if (personInfo.getFaceBound()!=null && personInfo.getFaceBound()){
                throw new Exception("人员照片已绑定到设备，不能再上传!");
            }

            String retFileUrl = OSSUtil.upload(ossConfig,"/person",photoName,photoFile.getInputStream());

            messageResult.setResult(true);
            messageResult.setData(retFileUrl);
            messageResult.setCode(200);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }

    @PostMapping("save")
    @ApiOperation(value="保存人员信息并将照片上传到终端")
    @ApiImplicitParams({
            @ApiImplicitParam(name="personId",value = "人员编号",required = true,paramType = "form"),
            @ApiImplicitParam(name="openId",value = "微信openId",required = true,paramType = "form"),
            @ApiImplicitParam(name="faceImageUrl",value = "照片地址",required = true,paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "form")
    })
    public MessageResult<PersonInfo> save(Long personId,String openId,String faceImageUrl,String token){
        MessageResult<PersonInfo> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(personId);

            if (personInfo==null){
                throw new Exception("人员信息不存在!");
            }

            if (personInfo.getFaceBound()!=null && personInfo.getFaceBound()){
                throw new Exception("人员照片已绑定到设备，不能再上传!");
            }

            //todo 更新人脸照片
            personInfo.setOpenId(openId);
            personInfo.setFaceImageUrl(faceImageUrl);
            personInfo.setUpdateTime(new Date());

            personInfoService.update(personInfo);

            //todo 加入到导入图片队列中
            rabbitTemplate.convertAndSend("importImageExchange","importImage",personId);

            messageResult.setData(personInfo);
            messageResult.setResult(true);

            messageResult.setCode(200);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="人员列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "姓名", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "idCard",value = "身份证", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "phone",value = "手机号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "query",dataType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<Map> pageList(
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="idCard",defaultValue="") String idCard,
            @RequestParam(value="phone",defaultValue="") String phone,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            String token,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            User user = userService.get(subject);

            if (user==null){
                throw new Exception("当前用户不是管理员！");
            }

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time", "asc"));

            if (StringUtils.isNotEmpty(name)) {
                searchParams.put("name", "%" + name + "%");
            }
            if (StringUtils.isNotEmpty(idCard)) {
                searchParams.put("idCard", "%" + idCard + "%");
            }
            if (StringUtils.isNotEmpty(phone)) {
                searchParams.put("phone", "%" + phone + "%");
            }

            searchParams.put("companyId", user.getCompanyId());

            Page<PersonInfo> page = personInfoService.pageSearch(searchParams, pageIndex, pageSize, true, sortList);

            msgResult.setResult(true);
            msgResult.setData(PojoUtils.pageWrapper(page));
        }
        catch (Exception e) {
            logger.error(e.getMessage(),e);
            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
        }



        return msgResult;
    }

    @PostMapping("add")
    @ApiOperation(value="添加人员信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name="name",value = "姓名",required = true,paramType = "form"),
            @ApiImplicitParam(name="phone",value = "电话",required = true,paramType = "form"),
            @ApiImplicitParam(name="faceImageUrl",value = "照片地址",required = true,paramType = "form"),
            @ApiImplicitParam(name="bindDevices",value = "绑定设备(,分隔)",required = true,paramType = "form"),
            @ApiImplicitParam(name="token",value = "令牌",paramType = "form"),
            @ApiImplicitParam(name="subject",value = "目标(不传)",paramType = "form")
    })
    public MessageResult<PersonInfo> add(String name,String phone,String faceImageUrl,String bindDevices,String token,@RequestAttribute String subject){
        MessageResult<PersonInfo> messageResult = new MessageResult<>();

        try {
            PersonInfo admin = personInfoService.get(Long.valueOf(subject));

            if (StringUtils.isEmpty(admin.getPopedom())
            && admin.getPopedom().indexOf("2")==-1){
                throw new Exception("当前用户不是管理员！");
            }

            PersonInfo personInfo = personInfoService.findByNameAndPhone(name, phone);
            if(personInfo == null) {
                personInfo = new PersonInfo();
                personInfo.setCompanyId(admin.getCompanyId());
                personInfo.setName(name);
                personInfo.setPhone(phone);
                personInfo.setFaceEnabled(true);
                personInfo.setFaceImageUrl(faceImageUrl);
                personInfo.setPopedom("1");
                personInfo.setDelFlag(false);
                personInfo.setCreateBy(String.valueOf(admin.getId()));
                personInfo.setCreateTime(new Date());

                int affectCount = personInfoService.insert(personInfo);

                personInfo = personInfoService.findByNameAndPhone(name, phone);

                //todo 关联设备
                String[] deviceIds = bindDevices.split(",");
                for(String deviceId : deviceIds){
                    PersonDeviceRelation personDeviceRelation = new PersonDeviceRelation();
                    personDeviceRelation.setId(UUID.randomUUID().toString());
                    personDeviceRelation.setDeviceId(deviceId);
                    personDeviceRelation.setPersonId(personInfo.getId());
                    personDeviceRelation.setDelFlag(false);
                    personDeviceRelation.setCreateBy(subject);
                    personDeviceRelation.setCreateTime(new Date());
                    personDeviceRelationService.insert(personDeviceRelation);
                }

                if(affectCount>0) {
                    //todo 加入到导入图片队列中
                    rabbitTemplate.convertAndSend("importImageExchange", "importImage", personInfo.getId());
                }

                messageResult.setData(personInfo);
                messageResult.setResult(true);
                messageResult.setCode(200);
            }
            else{
                messageResult.setResult(false);
                messageResult.setMessage("人员已存在。");
            }
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }
}
